*PROCESS USING(NOMAP,NOWARN),RENT,FLAG(ALIGN,CONT)
DBNUXTRM TITLE 'RMDS V2.3 Termination Exit'
         GBLC  &DBNMLST                 Macro LIST option
         GBLC  &DBNVERS                 Code version string
         SPACE 2
&DBNMLST SETC  'NO'
&DBNVERS SETC  '2.3'
         SPACE 2
DBNUXTRM CSECT ,
DBNUXTRM AMODE ANY
DBNUXTRM RMODE 24
         SPACE 2
**** DBNUXTRM:  RMDS V2.3 Termination Exit
*
*
* Classification:
*        Licensed Materials - Property of IBM.
*
*        (C) Copyright IBM Corp. 1998
*
*
* Function:
*        DBNUXTRM can be used to perform any special processing
*        that needs to be done at the end of a user session or
*        when an RMDS functional area is terminating.
*
*        DBNUXTRM is called once per viewer session as part of
*        session termination.  It is also called when the VTAM
*        viewer application is terminating.  For nonviewer
*        functions, DBNUXTRM is called when the job or task is
*        about to terminate.
*
*        See RMDS publication S544-5398 "Customization, Tuning and
*        Diagnosis" for the details on the function and operation
*        of this exit.
*
*        The logic contained in this sample implementation allows
*        termination to continue unless all of the following
*        conditions are satisfied:
*
*              . The terminating process is the Archiver.
*              . The Archiver is preparing to ABEND.
*
*        When these conditions are satisfied, ABEND avoidance is
*        attempted and the SYSOUT data set being processed is
*        placed in an OPER HOLD state.
*
*        As an aid in further problem determination, this exit
*        outputs the following messages via a multi-line WTO:
*
*              DBNUXTRM-01I <archiver name>
*                           ATTEMPTING RECOVERY FOR <job id>
*                           CLASS=<SYSOUT Class>  FORM=<form name>
*                           SPOOL DSN=<dsname>
*                           GROUPID=<JES output group id>
*                           ARCHIVE DSN=<dsname>
*                           ARCHIVE TYPE=<archive type>
*
*
* Attributes:
*        Reentrant
*        Refreshable
*
*
* Execution Environment:
*        Minimum Authorization:  Problem state, PSW key 8-15.
*                                Supervisor state, JES key when
*                                invoked by the Archiver.
*        Dispatchable Unit mode: Task
*        Cross Memory Mode:      PASN=HASN=SASN
*        Addressing Mode:        24- or 31-bit
*        ASC Mode:               Primary
*        Interrupt Status:       Fully enabled
*        Locks:                  No locks held
*
*
* System Dependencies:
*        1.  The ESA/390 Linkage Stack Facility as described in
*            SA22-7201, "ESA/390 Principles of Operation".
*
*
* Serialization Requirements:
*        None.
*
*
* Inputs:
*        1.  General Registers
*
*            R1:   Points to the invocation parameter list which
*                  is mapped as follows:
*
*                  +0:  A fullword pointer to a 4-byte string
*                       which identifies the caller of this
*                       exit:
*
*                       ACHV  DBNARCHV
*                       ALOD  DBNARLOD
*                       BCOM  DBNBPCOM
*                       BDTR  DBNBPDTR
*                       CV22  DBNCUV22
*                       MOLM  DBNMAOLM
*                       MPRG  DBNMAPRG
*                       VAPL  DBNVTAPL
*                       VCL3  DBNVTCL3
*                       VPRM  DBNVWPRM
*
*            R10:  Points to one of the following structures:
*
*                  Macro     DSECT     Structure Description
*                  ---------------------------------------------------
*                  DBNXACA   ACA       Archiver Communications Area
*                  DBNXBPCA  BPCA      Batch Print Communications Area
*                  DBNXMCA   MCA       Maintenance Communications Area
*                  DBNXSRCA  SRCA      Subroutine Communications Area
*                  DBNXVSB   VSB       Viewer Session Block
*
*            R13:  Points to a standard 18-word OS register save
*                  area.
*
*            R14:  Contains the address to which this module is
*                  to branch upon completion.
*
*            R15:  Contains the entry point address of this module.
*
*
* Outputs:
*        1.  General Registers
*
*            R0-R14:  The same as at entry, except as noted below.
*
*            R15:     Contains a return code.
*                     Return code values are defined as follows:
*
*                     0   Continue termination processing.
*
*                     4   Continue termination processing.
*
*                     5 or Higher
*                         If EVT field EVTMSG# is non-zero, then
*                         the corresponding installation defined
*                         message is issued.  Otherwise, an RMDS
*                         message is issued which reflects the
*                         returned contents of GPR0 and GPR1 (these
*                         are interpreted by RMDS as the failure
*                         Return and Reason codes, respectively).
*
*                         Except for the Viewer application and the
*                         Archiver program when the return code is
*                         12 (see below), the job or task is
*                         terminated.
*
*                     12  For the Archiver program only, if the
*                         termination was due to an unrecoverable
*                         error, the Archiver program continues
*                         with termination.
*
*                         If the termination was due to an RMDS
*                         detected error, the Archiver program
*                         continues processing SYSOUT.  In a JES3
*                         environment, the current SPOOL data set
*                         is dynamically deallocated, as specified
*                         by the values set by DBNUXTRM in R0 and
*                         R1:
*
*                           R0=0   Keep the SPOOL data set
*
*                           R0=4   Delete the SPOOL data set
*
*                           R0=8   Keep the SPOOL data set,
*                                  requeuing the data set to the
*                                  writer queue, using the class
*                                  specified in the fourth byte of
*                                  R1 (valid for data sets on the
*                                  JES3 HOLD queue only)
*
*                           R0=12  Keep the SPOOL data set on the
*                                  hold queue, changing the class
*                                  to the class specified in the
*                                  fourth byte of R1 (valid for
*                                  data sets on the JES3 HOLD queue
*                                  only)
*
*                         In a JES2 environment, DBNUXTRM must
*                         requeue the SPOOL data set or take
*                         whatever action is necessary to allow
*                         recovery to be performed.
*
*                         Note:  The termination user exit must
*                                reset the EVT fields EVTUXID,
*                                EVTMSG#, EVTMSGV1, EVTMSGV2,
*                                EVTMSGV3 and EVTUXREG if other
*                                user exits are dependent on these
*                                fields.
*
*
* Data Areas:
*        . CVT       MVS Communications Vector Table
*        . DBNXACA   RMDS Archiver Communications Area
*        . DBNXBPCA  RMDS Batch Print Communications Area
*        . DBNXCCA   RMDS Common Communications Area
*        . DBNXEVT   RMDS Exit Vector Table
*        . DBNXMCA   RMDS Maintenance Communications Area
*        . DBNXSRCA  RMDS Subroutine Communications Area
*        . DBNXVSB   RMDS Viewer Session Block
*        . IEFJSSIB  MVS Subsystem Identification Block
*        . IEFSSOBH  MVS Subsystem Options Block Header
*        . IEFSSSO   MVS Process SYSOUT SSOB Extension
*        . IEFTIOT1  MVS Task I/O Table
*        . IEZJSCB   MVS Job/Step Control Block
*        . IEZMGCR   MVS Command Scheduler Plist
*        . IHAPSA    MVS Prefixed Storage Area'
*        . IKJTCB    MVS Task Control Block'
*
*
* Exits (Normal):
*        Via PR  (Program Return)
*
*
* Exits (Abnormal):
*        None.
*
*
* External References:
*        None.
*
*
* Special Considerations:
*
*        Special Considerations when Invoked by the Archiver
*        ---------------------------------------------------
*
*        DBNUXTRM is invoked at normal termination of the Archiver
*        as well as when the Archiver is preparing to abnormally
*        terminate.
*
*        In situations where the Archiver is preparing to
*        abnormally terminate, DBNUXTRM may be able to perform
*        actions which will allow the Archiver to continue its
*        processing and not terminate via ABEND.
*
*        It is important to understand that DBNUXTRM *is not* an
*        Archiver ABEND recovery exit.  Rather, it is an Archiver
*        ABEND avoidance exit.  If an ABEND of any type occurs
*        within the Archiver, DBNUXTRM *is not* invoked.
*
*        The following ACA fields may be helpful to DBNUXTRM in
*        determining the nature of the failure/error condition for
*        which it is being invoked:
*
*          ACAERROR   If the failure condition was detected by an
*                     internal RMDS service routine, subordinate
*                     fields ACAERRR15, ACAERRG0, and ACAERRG1
*                     contain the image of general register R15,
*                     R0, and R1, respectively, upon return from
*                     the service routine.
*
*          ACASRMOD    If the failure condition was detected by an
*                      internal RMDS service routine, this field
*                      contains the service routine name.
*
*          ACAERMSG    Contains the identification number of the
*                      associated RMDS diagnostic message.
*
*          ACAIDCMG    May contain the identification number of the
*                      associated IDCAMS diagnostic message.
*
*        It is difficult for DBNUXTRM to determine if ACA fields
*        ACAIDCMG, ACAERROR, and ACASRMOD contain meaningful
*        information.  These fields are not reset by the Archiver
*        and, therefore, may contain residual information
*        pertaining to earlier conditions which are not relevant to
*        the situation for which DBNUXTRM is currently being
*        invoked.
*
*        IF RECOVER=Y has been specified in the RMDS system options
*        (DBNPARM) and if a "recoverable" failure (see the
*        Customization, Tuning and Diagnosis Guide for a description
*        of Archiver recoverable failures) occurs while archiving a
*        SEP=C report, then:
*
*          . DBNUXTRM is not invoked at the time the failure is
*            encountered.
*
*          . The Archiver attempts to re-process the spool file
*            which was being processed when the error was
*            encountered.
*
*          . If a second failure occurs, the Archiver prepares
*            to terminate via a U0003 ABEND.  DBNUXTRM is invoked
*            with ACA flag bits ACAFLG2.ACARCOVR and
*            ACAFLG3.ACAABEND set to 1. If DBNUXTRM wants to
*            prevent the Archiver from ABENDing, then DBNUXTRM
*            must set bit ACAFLG2.ACARCOVR to 0 and exit with
*            GPR15 set to 12 (X'0000000C').
*
*
* Module-wide Register Usage:
*        R12:  Code base.
*        R11:  Local work area base (DSECT WORKMAP).
*        R10:  RMDS ACA, BPCA, MCA, SRCA or VSB base.
*
*
* Patch Space:
*        128 bytes, double-word aligned starting at label PATCH.
*
*
* Change History:
*        21 Oct 1998  RJR  Created.
*
* --------------------------------------------------------------------
         TITLE 'Macro ICALL:    Internal Procedure CALL'
         MACRO ,
&LBL     ICALL  &TGT
.*
         AIF   ('&LBL' EQ '').LBLEND
&LBL     DC    0H'0'
.LBLEND  ANOP  ,
         AIF   ('&TGT' EQ '(R15)').LINK
         AIF   ('&TGT' EQ '(15)').LINK
         AIF   ('&TGT'(1,1) NE '(').GLA
         LR    R15,&TGT(1)              Set the target EPA
         AGO   .LINK
.GLA     ANOP  ,
         LA    R15,&TGT                 Set the target EPA
.LINK    ANOP  ,
         BAKR  0,R15                    Invoke the target proc
         MEXIT ,
         MEND  ,
         TITLE 'RMDS V2.3 Termination Exit'
* --------------------------------------------------------------------
*
*                       General Register Equates
*
* --------------------------------------------------------------------
         SPACE 2
R0       EQU   0
R1       EQU   1
R2       EQU   2
R3       EQU   3
R4       EQU   4
R5       EQU   5
R6       EQU   6
R7       EQU   7
R8       EQU   8
R9       EQU   9
R10      EQU   10
R11      EQU   11
R12      EQU   12
R13      EQU   13
R14      EQU   14
R15      EQU   15
         SPACE 2
* --------------------------------------------------------------------
*
*                       Access Register Equates
*
* --------------------------------------------------------------------
         SPACE 2
AR0      EQU   0
AR1      EQU   1
AR2      EQU   2
AR3      EQU   3
AR4      EQU   4
AR5      EQU   5
AR6      EQU   6
AR7      EQU   7
AR8      EQU   8
AR9      EQU   9
AR10     EQU   10
AR11     EQU   11
AR12     EQU   12
AR13     EQU   13
AR14     EQU   14
AR15     EQU   15
         EJECT ,
* --------------------------------------------------------------------
*
*  The following Equates define displacements into the standard
*  18-word OS register save area for each of the general registers,
*  as well as the forward and backward save area block links.
*
* --------------------------------------------------------------------
         SPACE 2
SAVEBACK EQU   4,4                      Link to previous S.A.
SAVEFWRD EQU   8,4                      Link to next S.A.
*
SAVER14  EQU   12,4
SAVER15  EQU   16,4
SAVER0   EQU   20,4
SAVER1   EQU   24,4
SAVER2   EQU   28,4
SAVER3   EQU   32,4
SAVER4   EQU   36,4
SAVER5   EQU   40,4
SAVER6   EQU   44,4
SAVER7   EQU   48,4
SAVER8   EQU   52,4
SAVER9   EQU   56,4
SAVER10  EQU   60,4
SAVER11  EQU   64,4
SAVER12  EQU   68,4
         EJECT ,
* --------------------------------------------------------------------
*
* The following equates define the valid Return Code values which
* may be returned (in GPR R15) from this exit.
*
* --------------------------------------------------------------------
         SPACE 2
RCCTERM  EQU   0                        Common to all invocations:
*
*                                       Termination processing is
*                                       to continue.
         SPACE 2
RCCERR   EQU   5                        Common to all invocations:
*
*                                       A fatal error condition
*                                       has occurred.
         SPACE 2
RCARCVR  EQU   12                       For the Archiver only:
*
*                                       DBNUXTRM has performed
*                                       recovery action.  The
*                                       Archiver is to attempt
*                                       to continue processing.
         SPACE 3
* --------------------------------------------------------------------
*
* Miscellaneous Equates
*
* --------------------------------------------------------------------
         SPACE 2
ALL      EQU   X'FF'
ZERO     EQU   0
BLANK    EQU   C' '
COMMA    EQU   C','
EOL      EQU   X'80'                    Standard End-of-List flag.
         EJECT ,
         BAKR  R14,0                    Stack the entry state.
         STM   R14,R12,SAVER14(R13)     Save the entry regs
         BASR  R12,0                    Establish a local base.
         LA    R15,(*-DBNUXTRM)(,0)     Offset from CSECT start.
         LA    R12,0(,R12)              Purify the base value.
         SLR   R12,R15                  Set a full CSECT base.
*
         XR    R15,R15                  * Set the CSECT base
         SAR   AR12,R15                 * Access Register.
*
         USING DBNUXTRM,R12
         USING PSA,R0
*
         LA    R14,IDEND                * Set the AMODE-31 switch
         O     R14,AMODE31              * target value and switch
         BSM   0,R14                    * to AMODE-31
*
         DC    AL1(IDEND-*-1)           ID string length.
MODID    DC    CL8'DBNUXTRM'
         DC    CL1' '
         DC    C'V&DBNVERS'
         DC    CL1' '
         DC    C'&SYSDATC &SYSTIME'
         DC    C' COPYRIGHT (C) IBM CORP. 1998'
         DC    0F'0'
         DC    A(MODSIZE)               CSECT length
*
IDEND    DC    0H'0'
         EJECT ,
* -------------------------------------------------------------------
*
* Termination is allowed to continue unless all of the following
* conditions are satisfied:
*
*        . The terminating process is the Archiver.
*
*        . The Archiver is preparing to ABEND.
*
* -------------------------------------------------------------------
         SPACE 2
         USING ACA,R10
*
         LR    R3,R1                    Save the invocation
*                                       plist pointer.
*
         XR    R0,R0                    * Zero the error feedback
         XR    R1,R1                    * registers.
*
         LA    R15,RCCTERM(,0)          Assume there is nothing
*                                       to do.
*
         CLC   IDACA,ACAEYE             Is the exit being invoked
*                                       by the Archiver or Loader.
*
         BNE   LEAVE                    Br if not.
*
         TM    ACAFLAG2,ACAENV          * Do nothing if we are
         BNZ   LEAVE                    * not in the Archiver
*                                       * environment.
*
         TM    ACAFLG3,ACAABEND         * Do nothing if the
         BZ    LEAVE                    * Archiver is not
*                                       * preparing to ABEND.
         EJECT ,
* -------------------------------------------------------------------
*
*  Obtain and initialize a workarea local to this module.
*
* -------------------------------------------------------------------
         SPACE 2
         L     R2,SIZEWORK              Load the wkarea length
         SPACE 2
         STORAGE  OBTAIN,               Obtain our workarea            +
               COND=NO,                 Unconditional request          +
               LENGTH=(R2),             Required length in R2          +
               SP=WKSPIQ,               Required VSM subpool Id        +
               LOC=ANY                  Any storage residency
         EJECT ,
         LAE   R11,0(R1,0)              Set the work area base
*                                       and its associated AR.
         USING WORKMAP,R11
*
         LR    R0,R11                   Set the MVCL 'To' adrs
         LR    R1,R2                    Set the MVCL 'To' length
         XR    R14,R14                  Zero the MVCL 'From' adrs
         XR    R15,R15                  Set the pad character and
*                                       MVCL 'From' length.
*
         MVCL  R0,R14                   Zero the work area
*
         ST    R2,WKDESC                Fill in the wkarea length.
         MVI   WKDESC,WKSPIQ            Fill in the wkarea VSM SPID.
         MVC   WKBLKID,NMWORK           Fill in the wkarea blk ID
         MVC   WKOWNER,MODID            Fill in the owner ID
*
         ST    R3,WKPARMP               Save the pointer to the
*                                       invocation parameters list.
*
         LA    R15,WKFLRSA              Set the ptr to the local
*                                       register save area.
*
         ST    R13,SAVEBACK(,R15)       Ptr to old RSA from new
         ST    R15,SAVEFWRD(,R13)       Ptr to new RSA from old
         LR    R13,R15                  Set the current RSA base
*
         ST    R12,WKBASE1              Save the mainline base
*                                       reg value in the work
*                                       area.
         EJECT ,
         TIME  DEC,                     Obtain the system timestamp    +
               WKTSTAMP,                Feedback area                  +
               ZONE=LT,                 Obtain Local time              +
               DATETYPE=MMDDYYYY,       Desired date format            +
               LINKAGE=SYSTEM,                                         +
               MF=(E,WKLFTIME)
         SPACE 2
         L     R2,WKDATEJ               Fetch the returned date
         XC    WKTSTAMP,WKTSTAMP        Zero the TIME feedback area
         SPACE 2
         TIME  DEC,                     Obtain the system timestamp    +
               WKTSTAMP,                Feedback area                  +
               ZONE=LT,                 Obtain Local time              +
               DATETYPE=YYYYDDD,        Desired date format            +
               LINKAGE=SYSTEM,                                         +
               MF=(E,WKLFTIME)
         SPACE 2
         ST    R2,WKDATEG               Save the gregorian date
         EJECT ,
         L     R14,ACACCA@              Fetch the RMDS CCA ptr.
         USING CCA,R14
*
         L     R15,CCAEVT@              Fetch the RMDS EVT ptr.
         USING EVT,R15
*
         XC    EVTMSG#,EVTMSG#          Indicate no error message
*                                       is to be issued upon
*                                       return.
*
         DROP  R15                      EVT base
         DROP  R14                      CCA base
         SPACE 2
         ICALL  HARCH                   Perform Archiver recovery
         EJECT ,
* --------------------------------------------------------------------
*
*  Free the local work area and exit.
*
* --------------------------------------------------------------------
         SPACE 2
         PUSH  USING
*
         LR    R2,R15                   * Save the mainline
         LR    R3,R0                    * completion code and
         LR    R8,R1                    * the function return
*                                       * and reason codes.
*
         L     R13,WKFLRSA+SAVEBACK     Fetch the entry RSA ptr
*
         XR    R0,R0                    Zero the FREEMAIN len reg
         ICM   R0,B'0111',WKDESC+1      Fetch the work area length
*
         XR    R15,R15                  Zero the FREEMAIN SPID reg
         IC    R15,WKDESC               Fetch the work area SPID
         LA    R1,WORKMAP               Ptr to area to free
*
         DROP  R11                      WORKMAP base
         SPACE 2
         STORAGE  RELEASE,              Free the work area             +
               COND=NO,                 Unconditional request          +
               LENGTH=(0),              Area length in R0              +
               ADDR=(1),                Area address in R1             +
               SP=(15)                  VSM Subpool Id in R15
         EJECT ,
         LM    R0,R1,SAVER0(R13)        Restore R0 and R1 as they
*                                       were upon entry.
*
         LR    R15,R2                   Set the mainline ret code.
*
         LA    R14,RCCERR(,0)           * Has a fatal error been
         CR    R15,R14                  * encountered --
*
         BL    LEAVE                    Br if not.
*
         LR    R0,R3                    * Else set the corresponding
         LR    R1,R8                    * error feedback info regs.
         SPACE 2
LEAVE    DC    0H'0'
         PR    ,                        Exit
*
         POP   USING
         TITLE 'HARCH:    Perform Archiver Recovery'
****  HARCH:    Perform Archiver Recovery
*
*
* Function:
*        This procedure is invoked to attempt to avoid an Archiver
*        ABEND.
*
*        The logic contained in the sample exit:
*
*              . Outputs a status message via WTO to aid in problem
*                determination;
*
*              . Attempts to place the SYSOUT data set being
*                processed into an OPER HOLD state;
*
*              . Sets the return code indicating that the Archiver
*                is to continue its processing and avoid the ABEND.
*
* Inputs:
*        None directly.
*
*
* Outputs (normal):
*        R0:   Zero
*        R1:   Zero
*
*        R15:  RCARCVR
*
*
* Exits (error):
*        None.
*
*
* Special Considerations:
*        None.
*
*
* Linkage:
*        Entered via the ICALL macro.
*        Normal Exit via Program Return (PR).
*
****
         SPACE 2
HARCH    DC    0H'0'
         PUSH  USING
         EJECT ,
         MVI   WKCMDBUF,BLANK           Blank the MGCR buffer.
         MVC   WKCMDBUF+1(L'WKCMDBUF-1),WKCMDBUF
*
         USING MGCRPL,WKCMDBUF
*
         LA    R14,WKM1TEXT             Set the MVCL 'To' adrs.
         LA    R0,SKM1TEXT              Set the MVCL 'From' adrs.
         LA    R1,SKM1LEN(,0)           Set the MVCL 'From' length.
         LR    R15,R1                   Set the MVCL 'To' length
*
         MVCL  R14,R0                   Copy the message line
*                                       templates to the work
*                                       area.
*
         MVC   WKM1WTO(LSKM1WTO),SKM1WTO
*                                       Copy the skeletal WTO
*                                       plist to the work area.
         EJECT ,
* --------------------------------------------------------------------
*
* Place the Archiver's Job Name and JES assigned Job ID into the
* message buffer.
*
* --------------------------------------------------------------------
         SPACE 2
         L     R15,PSATOLD              Fetch the current TCB ptr.
         USING TCB,R15
*
         L     R14,TCBTIO               Fetch the TIOT ptr.
         USING IEFTIOT1,R14
*
         MVC   WKM1ARJN,TIOCNJOB        Place the Archiver Job
*                                       Name into the msg bufr.
*
         DROP  R14                      IEFTIOT1 base
*
         XR    R14,R14                  * Fetch the JSCB ptr.
         ICM   R14,B'0111',TCBJSCBB     *
*
         USING IEZJSCB,R14
*
         L     R14,JSCBSSIB             Fetch the ptr to the
*                                       Life-of-Job SSIB.
*
         DROP  R14                      IEZJSCB base
         USING SSIB,R14
*
         MVC   WKM1ARJI,SSIBJBID        Place the Archiver Job.
*                                       ID into the msg bufr.
*
         DROP  R14                      SSIB base
         DROP  R15                      TCB base
         EJECT ,
* --------------------------------------------------------------------
*
* The JES2 command which is used to place the SPOOL data set into
* an OPER HOLD state is of the form:
*
*        $TO<jobid>,OUTGRP=<groupid>,HOLD=OPER
*
* The OUTGRP operand is positional and if present, must immediately
* follow the Job ID string.  If JES2 has not supplied the Output
* Group ID in the PSO SSOB Extension then all output groups for the
* Job are placed in an OPER HOLD state and the command issued is of
* the form:
*
*        $TO<jobid>,ALL,HOLD=OPER
*
* --------------------------------------------------------------------
         SPACE 2
         MVC   MGCRTEXT(L'TX$T),TX$T    Build the JES2 command
*                                       verb.
*
         L     R15,ACASSOB@             Fetch the SSOB ptr.
         USING SSOB,R15
*
         L     R15,SSOBINDV             Fetch the ptr to the
*                                       PSO SSOB extension.
*
         DROP  R15                      SSOB header base
         USING IEFSSSO,R15
*
         MVC   WKM1UJN,ACAJBN           * Place the ID info for
         MVC   WKM1UJI,ACAJBI           * the Job being processed
*                                       * into the msg buffer.
*
         MVC   MGCRTEXT+L'TX$T(L'ACAJBI),ACAJBI
*                                       Place the ID of the Job
*                                       being processed into the
*                                       command string.
*
         LA    R14,MGCRTEXT+L'TX$T+L'ACAJBI
*                                       Point beyond the Job ID
*                                       in the command bufr.
*
         MVI   0(R14),COMMA             Put in an oprnd delimiter.
*
         LA    R14,1(,R14)              Obtain the location of the
*                                       next command operand.
*
         LR    R0,R14                   Temp save the command
*                                       buffer cursor.
*
         MVC   0(L'TXALL,R14),TXALL     Place the 'ALL,' text in
*                                       the command buffer.
*
         LA    R14,L'TXALL(,R14)        Advance the command
*                                       buffer cursor.
*
         TM    SSSOFLG2,SSSOPSEE        Is the Extended PSO SSOB
*                                       Extension present --
*
         BNO   HAR400                   Br if not.
*
         TM    SSSOFLG5,SSSOGNVA        Has JES2 provided the
*                                       Output Group ID --
*
         BNO   HAR400                   Br if not
*
         LR    R14,R0                   Restore the command
*                                       buffer cursor.
*
         MVC   0(L'TXGROUP,R14),TXGROUP
*                                       Place the OUTGRP operand
*                                       string in the command
*                                       buffer.
*
         LA    R14,L'TXGROUP(,R14)      Advance the command
*                                       buffer cursor.
*
         MVC   WKM1SGID,SSSOOGNM        Place the Output Group ID
*                                       in the message buffer.
*
         MVC   0(L'SSSOOGNM,R14),SSSOOGNM
*                                       Place the Output Group ID
*                                       in the command buffer.
*
         LA    R14,L'SSSOOGNM(,R14)     Point to the first byte
*                                       beyond the Output Group
*                                       ID string in the command
*                                       buffer.
*
HAR300   DC    0H'0'
         CLI   0(R14),BLANK             Look for the righmost
*                                       non-blank of the Group
*                                       Id string.
*
         BNE   HAR340                   Br if found.
         BCT   R14,HAR300               Else continue the scan.
*
HAR340   DC    0H'0'
         MVI   1(R14),COMMA             Put in an oprnd delimiter.
*
         LA    R14,2(,R14)              Obtain the location of the
*                                       next command operand.
*
HAR400   DC    0H'0'
         MVC   0(L'TXHOLD,R14),TXHOLD   Place the 'HOLD=OPER' text
*                                       in the command buffer.
*
         LA    R14,L'TXHOLD(,R14)       Point to the first byte
*                                       beyond the HOLD=OPER
*                                       string in the command
*                                       buffer.
*
         LA    R0,MGCRPL                * Compute the effective
         SLR   R14,R0                   * command bufr length.
*
         SLL   R14,16                   * Build the Control
         STCM  R14,B'1111',MGCRPL       * portion of the
*                                       * command buffer.
*
         MVC   WKM1SCLS,SSSOCLAS        Place the SYSOUT Class
*                                       of the SPOOL file being
*                                       processed in the message
*                                       buffer.
*
         MVC   WKM1SFRM,ACAFRM          Place the effective FORM
*                                       name associated with the
*                                       SPOOL file into the msg
*                                       buffer.
*
         TRT   ACAARDSN,DSNTBL          Does the archive DSN
*                                       string look reasonable--
*
         BNZ   HAR500                   Br if not.
*
         CLI   ACAARDSN,BLANK           * Br if the archive DSN
         BE    HAR500                   * string starts with a
*                                       * blank.
*
         MVC   WKM1ADSN,ACAARDSN        Place the archive DSN
*                                       into the msg buffer.
*
HAR500   DC    0H'0'
         MVC   WKM1SDSN,SSSODSN         Place the DSN of the SPOOL
*                                       data set being processed
*                                       into the message buffer.
*
         DROP  R15                      IEFSSSO base
         EJECT ,
* --------------------------------------------------------------------
*
* The archive data set type is determined by looping through a
* table containing entries corresponding to the bit field contained
* in ACA field ACARTYPE.
*
* --------------------------------------------------------------------
         SPACE 2
         LA    R14,TBLART               Point to the Archive Type
*                                       lookup table.
*
         LA    R0,TBLARTN(,0)           Load the tbl entry count.
*
HAR800   DC    0H'0'
         IC    R15,0(,R14)              Fetch the archive type
*                                       bit test mask.
*
         EX    R15,HARTART              Perform the test.
         BO    HAR840                   Br if found.
         LA    R14,L'TBLART(,R14)       Else bump the table ptr.
         BCT   R0,HAR800                And continue searching.
*
HAR840   DC    0H'0'
         MVC   WKM1ATYP,1(R14)          Place the Archive Type
*                                       string in the message
*                                       buffer.
         EJECT ,
* --------------------------------------------------------------------
*
* Output the multi-line status message via WTO
*
* --------------------------------------------------------------------
         SPACE 2
         XR    R0,R0                    Zero the WTO MSGID reg
         SPACE 2
         WTO   TEXT=((WKM1CTL),         Control Line                   +
               (WKM1L1),                Data Line 1                    +
               (WKM1L2),                Data Line 2                    +
               (WKM1L3),                Data Line 3                    +
               (WKM1L4),                Data Line 4                    +
               (WKM1L5),                Data Line 5                    +
               (WKM1L6)),               Data/End Line 6                +
               MF=(E,WKM1WTO)
         EJECT ,
* --------------------------------------------------------------------
*
* Issue the JES2 $TO command and delay for 1 second to allow JES
* time to process it.
*
* --------------------------------------------------------------------
         SPACE 2
         XR    R0,R0                    GPR0 must be zero.
         LA    R1,MGCRPL                Point to the command bufr
         SPACE 2
         MGCR  (1)                      Issue the command
         SPACE 2
         LA    R0,(1*100)(,0)           * Set the interval to
         ST    R0,WKTIMVAL              * 1 second.
         SPACE 2
         STIMERM  SET,                  Delay a bit                    +
               ID=WKTIMRID,             Request ID return area         +
               BINTVL=WKTIMVAL,         Timer interval area            +
               WAIT=YES,                Wait for interval expiration   +
               MF=(E,WKSTIMER)
         EJECT ,
* --------------------------------------------------------------------
*
* Set the return code to RCARCVR indicating that the Archiver is
* to attempt to continue its processing.
*
* --------------------------------------------------------------------
         SPACE 2
         NI    ACAFLG2,ALL-ACARCOVR     Reset the archiver
*                                       Recovery Attempted
*                                       flag.
*
         XR    R0,R0                    * Zero the error feedback
         XR    R1,R1                    * registers.
*
         LA    R15,RCARCVR(,0)          Set the return code which
*                                       indicates the Archiver is
*                                       to attempt to continue.
*
         PR    ,                        And exit.
         SPACE 3
* --------------------------------------------------------------------
*
* The following instruction is the object of an EXECUTE.
*
* --------------------------------------------------------------------

HARTART  TM    ACARTYPE,*-*             Test the Archive Type
*                                       bit field.
*
         POP   USING
*
*
         DROP  ,
         TITLE 'Patch Area and Literal Pool Constants'
* /////////////////////////////////////////////////////////////////////
* /                                                                   /
* /                            Patch Area                             /
* /                                                                   /
* /////////////////////////////////////////////////////////////////////
         SPACE 2
         PUSH  USING
         DROP  ,
         USING DBNUXTRM,R12
*
         DC    0D'0'
         DC    CL8'PATCH'
PATCH    DC    128S(*)                  Patch area
         POP   USING
         SPACE 3
* /////////////////////////////////////////////////////////////////////
* /                                                                   /
* /                     Literal Pool Constants                        /
* /                                                                   /
* /////////////////////////////////////////////////////////////////////
         SPACE 2
         DC    0D'0'
         SPACE 1
         LTORG ,
         TITLE 'Miscellaneous Constants'
* /////////////////////////////////////////////////////////////////////
* /                                                                   /
* /                    Miscellaneous Constants                        /
* /                                                                   /
* /////////////////////////////////////////////////////////////////////
         SPACE 2
AMODE31  DC    A(X'80000000')           A mask used to set or
*                                       test the AMODE-31 bit
*                                       of a fullword pointer.
*
SIZEWORK DC    A(WKLEN)                 Work area length.
*
NMWORK   DC    CL8'WORKMAP'             Local work area ID string.
*
BLANKS   DC    CL8' '
*
IDACA    DC    CL4'ACA'                 The ID string of the
*                                       ACA structure.
*
TXGROUP  DC    C'OUTGRP='
TXHOLD   DC    C'HOLD=OPER'
TX$T     DC    C'$TO'
TXALL    DC    C'ALL,'
         SPACE 2
* --------------------------------------------------------------------
*
* The following table is used to map the archive data set type
* contained in ACA bit field ACARTYPE to a corresponding string.
*
* --------------------------------------------------------------------
         SPACE 2
TBLART   EQU   *,5                      Table origin and element
*                                       length attribute.
*
         DC    AL1(ACARQSAM),CL4'QSAM'
         DC    AL1(ACARVSAM),CL4'VSAM'
         DC    AL1(ACAROAM),CL4'OAM'
         DC    AL1(ACARLDS),CL4'LDS'
*
TBLARTN  EQU   (*-TBLART)/(L'TBLART)    Number of elements.
*
         DC    AL1(0),CL4'----'         Element used when the
*                                       archive data set type
*                                       is unknown.
         EJECT ,
* --------------------------------------------------------------------
*
* The following translate table is used to validate Data Set Name
* strings.  The table considers as valid the following subsets as
* described in "OS/390 V2R4.0 MVS JCL Reference" (GC28-1757):
*
*        Alphanumeric
*        National
*        Special
*
* --------------------------------------------------------------------
         SPACE 2
DSNTBL   EQU   *,256
         DC    256XL1'FF'
*
         ORG   DSNTBL+C' '              Blank
         DC    AL1(0)
*
         ORG   DSNTBL+C'A'              Upper case A:I
         DC    9AL1(0)
*
         ORG   DSNTBL+C'J'              Upper case J:R
         DC    9AL1(0)
*
         ORG   DSNTBL+C'S'              Upper case S:Z
         DC    8AL1(0)
*
         ORG   DSNTBL+C'0'              Digits 0:9
         DC    10AL1(0)
*
         ORG   DSNTBL+X'5B'             Dollar Sign
         DC    AL1(0)
*
         ORG   DSNTBL+X'7B'             Pound Sign and At Sign
         DC    2AL1(0)
*
         ORG   DSNTBL+C'.'              Period
         DC    AL1(0)
*
         ORG   DSNTBL+C','              Comma
         DC    AL1(0)
*
         ORG   DSNTBL+C''''             Apostrophe
         DC    AL1(0)
*
         ORG   DSNTBL+C'&&'             Ampersand
         DC    AL1(0)
*
         ORG   DSNTBL+C'+'              Plus Sign
         DC    AL1(0)
*
         ORG   DSNTBL+C'-'              Minus Sign
         DC    AL1(0)
*
         ORG   DSNTBL+C'='              Equal Sign
         DC    AL1(0)
*
         ORG   DSNTBL+C'('              Left Paren
         DC    AL1(0)
*
         ORG   DSNTBL+C')'              Right Paren
         DC    AL1(0)
*
         ORG   DSNTBL+C'*'              Asterisk
         DC    AL1(0)
*
         ORG   DSNTBL+L'DSNTBL
         EJECT ,
* --------------------------------------------------------------------
*
* The following constants serve as the templates for the message
* segments for the multi-line WTO:
*
*        DBNUXTRM-01I <archiver name>
*                     ATTEMPTING RECOVERY FOR <job id>.
*                     CLASS=<SYSOUT Class>  FORM=<form name>
*                     SPOOL DSN=<dsname>
*                     GROUPID=<JES output group id>
*                     ARCHIVE DSN=<dsname>
*                     ARCHIVE TYPE=<archive type>
*
* --------------------------------------------------------------------
         SPACE 2
         DC    0D'0'
         SPACE 2
SKM1WTO  WTO   TEXT=((,C),              Control Line                   +
               (,D),                    Data Line 1                    +
               (,D),                    Data Line 2                    +
               (,D),                    Data Line 3                    +
               (,D),                    Data Line 4                    +
               (,D),                    Data Line 5                    +
               (,DE)),                  Data/End Line 6                +
               ROUTCDE=(1,10),                                         +
               DESC=(3,9),                                             +
               MF=L
         SPACE 2
LSKM1WTO EQU   *-SKM1WTO                WTO plist length
         EJECT ,
SKM1TEXT DC    0D'0'                    Origin of the skeletal
*                                       message text lines.
*
         DC    AL2(SKM1EC-*-2)          Line length.
         DC    C'DBNUXTRM-01I '
         DC    CL8' '                   Archiver Job Name.
         DC    CL1' '
         DC    CL8' '                   Archiver Job ID.
         DC    CL1' '
SKM1EC   EQU   *
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 1 area
         DC    AL2(SKM1E1-*-2)          Line length.
         DC    C'ATTEMPTING RECOVERY FOR '
         DC    CL8' '                   Job Name being processed.
         DC    CL1' '
         DC    CL8' '                   Job ID being processed.
         DC    CL1' '
SKM1E1   EQU   *
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 2 area
         DC    AL2(SKM1E2-*-2)          Line length.
         DC    C'CLASS='
         DC    CL1' '                   SYSOUT Class
         DC    CL2' '
         DC    C'FORM='
         DC    CL8' '                   FORM name
         DC    CL1' '
SKM1E2   EQU   *
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 3 area
         DC    AL2(SKM1E3-*-2)          Line length.
         DC    C'SPOOL DSN='
         DC    CL44' '                  SPOOL data set name.
         DC    CL1' '
SKM1E3   EQU   *
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 4 area
         DC    AL2(SKM1E4-*-2)          Line length.
         DC    C'GROUPID='
         DC    CL26'*** Not Available ***'
*                                       JES SPOOL data set
*                                       Output Group ID.
*
         DC    CL1' '
SKM1E4   EQU   *
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 5 area
         DC    AL2(SKM1E5-*-2)          Line length.
         DC    C'ARCHIVE DSN='
         DC    CL44'*** Not Available ***'
*                                       Archive DSNAME
         DC    CL1' '
SKM1E5   EQU   *
*
* --------------------------------------------------------------------
         DC    0F'0'                    Data Line 6 area
         DC    AL2(SKM1E6-*-2)          Line length.
         DC    C'ARCHIVE TYPE='
         DC    CL4' '                   Archive data set type.
         DC    CL1' '
SKM1E6   EQU   *
* --------------------------------------------------------------------
SKM1LEN  EQU   *-SKM1TEXT               Total message length
         SPACE 3
         DC    0D'0'
MODSIZE  EQU   *-DBNUXTRM               Total CSECT length
         TITLE 'CVT:       MVS Communications Vector Table'
         CVT   DSECT=YES,                                              +
               LIST=&DBNMLST
*
         TITLE 'DBNXACA:   RMDS Archiver Communications Area'
         DBNXACA  ,
*
         TITLE 'DBNXBPCA:  RMDS Batch Print Communications Area'
         DBNXBPCA ,
*
         TITLE 'DBNXCCA:   RMDS Common Communications Area'
         DBNXCCA  ,
*
         TITLE 'DBNXEVT:   RMDS Exit Vector Table'
         DBNXEVT  ,
*
         TITLE 'DBNXMCA:   RMDS Maintenance Communications Area'
         DBNXMCA  ,
*
         TITLE 'DBNXSRCA:  RMDS Subroutine Communications Area'
         DBNXSRCA  ,
*
         TITLE 'DBNXVSB:   RMDS Viewer Session Block'
         DBNXVSB  ,
*
         TITLE 'IEFJSSIB:  MVS Subsystem Identification Block'
         IEFJSSIB  ,
*
         TITLE 'IEFSSOBH:  MVS Subsystem Options Block Header'
         IEFSSOBH  ,
*
         TITLE 'IEFSSSO:   MVS Process SYSOUT SSOB Extension'
IEFSSSO  DSECT ,
SSOBGN   EQU   *
*
         IEFSSSO  SOEXT=YES
*
         TITLE 'IEFTIOT1:  MVS Task I/O Table'
IEFTIOT1 DSECT ,
         IEFTIOT1  ,
*
         TITLE 'IEZJSCB:   MVS Job/Step Control Block'
         IEZJSCB  ,
*
         TITLE 'IEZMGCR:   MVS Command Scheduler Plist'
         IEZMGCR  DSECT=YES
*
         TITLE 'IHAPSA:    MVS Prefixed Storage Area'
         IHAPSA  DSECT=YES,                                            +
               LIST=&DBNMLST
*
         TITLE 'IKJTCB:    MVS Task Control Block'
         IKJTCB  DSECT=YES,                                            +
               LIST=&DBNMLST
*
         TITLE 'WORKMAP:  Local Work Area Mapping'
WORKMAP  DSECT ,
         SPACE 1
***  WORKMAP:  Local Work Area Mapping
*
*
* Function:
*        Maps the working storage area obtained and used by
*        this module.
*
*
* Location:
*        Obtained via STORAGE OBTAIN from the VSM subpool
*        identified by the 'WKSPIQ' symbol.
*
*
* --------------------------------------------------------------------
         SPACE 3
WKSPIQ   EQU   0                        Workarea VSM subpool ID:
*                                       Job-Step protect key,
*                                       Pageable,
*                                       Private Area,
*                                       Fetch protected,
*                                       Auto-free at End-of-Task.
         SPACE 2
WKBLKID  DC    CL8'WORKMAP'             Data area ID to aid in
*                                       problem determination.
         SPACE 2
WKDESC   DC    0F'0'                    Data area descriptor:
WKSPID   DC    AL1(WKSPIQ)              VSM subpool id
WKSIZE   DC    AL3(WKLEN)               Storage area length
         SPACE 2
WKOWNER  DC    CL8' '                   CSECT name of the owner
*                                       of this structure.
         SPACE 2
WKTSTAMP  EQU   *,16                    Data area creation time
*                                       stamp.
*
WKTIME   DC    XL8'00'                  Time of day in unsigned
*                                       packed decimal format:
*                                       hhmmssuuuuuu0000.
*
WKDATEJ  DC    XL4'00'                  Julian date in unsigned
*                                       packed decimal format:
*                                       0yyyyddd.
*
WKDATEG  DC    XL4'00'                  Gregorian date in unsigned
*                                       packed decimal format:
*                                       mmddyyyy.
*
         SPACE 2
WKPARMP  DC    A(0)                     The plist pointer value
*                                       at entry to this module.
*
WKDBLA   DC    D'0'                     * Scratch pad.
WKDBLB   DC    D'0'                     *
WKDBLC   DC    D'0'                     *
*
WKFLRSA  DC    18F'0'                   A standard 18-word OS
*                                       GPR Save Area.
*
WKBASE1  DC    A(0)                     The mainline base value.
         SPACE 2
WKLFTIME TIME  LINKAGE=SYSTEM,          Skeletal TIME plist            +
               MF=L
         SPACE 2
         DC    0F'0'
WKCMDBUF DC    XL128'00'                The MGCR command buffer
         SPACE 2
WKSTIMER STIMERM  SET,                  Skeletal STIMERM plist         +
               MF=L
         SPACE 2
WKTIMRID DC    A(0)                     The STIMERM request ID.
*
WKTIMVAL DC    A(0)                     The STIMERM interval
*                                       value in binary hundreths
*                                       of seconds.
         EJECT ,
* --------------------------------------------------------------------
*
* The following areas are used to construct the message segments
* for the multi-line WTO:
*
*        DBNUXTRM-01I <archiver name>
*                     ATTEMPTING RECOVERY FOR <job id>.
*                     CLASS=<SYSOUT Class>  FORM=<form name>
*                     SPOOL DSN=<dsname>
*                     GROUPID=<JES output group id>
*                     ARCHIVE DSN=<dsname>
*                     ARCHIVE TYPE=<archive type>
*
* --------------------------------------------------------------------
         SPACE 2
WKM1WTO  WTO   TEXT=((,C),              Control Line                   +
               (,D),                    Data Line 1                    +
               (,D),                    Data Line 2                    +
               (,D),                    Data Line 3                    +
               (,D),                    Data Line 4                    +
               (,D),                    Data Line 5                    +
               (,DE)),                  Data/End Line 6                +
               ROUTCDE=(1,10),                                         +
               DESC=(3,9),                                             +
               MF=L
         EJECT ,
WKM1TEXT DC    0D'0'                    Control Line area
WKM1CTL  DC    AL2(0)                   Line length.
         DC    C'DBNUXTRM-01I '
WKM1ARJN DC    CL8' '                   Archiver Job Name.
         DC    CL1' '
WKM1ARJI DC    CL8' '                   Archiver Job ID.
         DC    CL1' '
* --------------------------------------------------------------------
WKM1L1   DC    0F'0'                    Data Line 1 area
         DC    AL2(0)                   Line length.
         DC    C'ATTEMPTING RECOVERY FOR '
WKM1UJN  DC    CL8' '                   Job Name being processed.
         DC    CL1' '
WKM1UJI  DC    CL8' '                   Job ID being processed.
         DC    CL1' '
* --------------------------------------------------------------------
WKM1L2   DC    0F'0'                    Data Line 2 area
         DC    AL2(0)                   Line length.
         DC    C'CLASS='
WKM1SCLS DC    CL1' '                   SYSOUT Class
         DC    CL2' '
         DC    C'FORM='
WKM1SFRM DC    CL8' '                   FORM name
         DC    CL1' '
* --------------------------------------------------------------------
WKM1L3   DC    0F'0'                    Data Line 3 area
         DC    AL2(0)                   Line length.
         DC    C'SPOOL DSN='
WKM1SDSN DC    CL44' '                  SPOOL data set name.
         DC    CL1' '
* --------------------------------------------------------------------
WKM1L4   DC    0F'0'                    Data Line 4 area
         DC    AL2(0)                   Line length.
         DC    C'GROUPID='
WKM1SGID DC    CL26' '                  JES SPOOL data set
*                                       Output Group ID.
*
         DC    CL1' '
* --------------------------------------------------------------------
WKM1L5   DC    0F'0'                    Data Line 5 area
         DC    AL2(0)                   Line length.
         DC    C'ARCHIVE DSN='
WKM1ADSN DC    CL44' '                  Archive data set name.
         DC    CL1' '
*
* --------------------------------------------------------------------
WKM1L6   DC    0F'0'                    Data Line 6 area
         DC    AL2(0)                   Line length.
         DC    C'ARCHIVE TYPE='
WKM1ATYP DC    CL4' '                   Archive data set type.
         DC    CL1' '
* --------------------------------------------------------------------
         DC    0D'0'
WKLEN    EQU   *-WORKMAP                Total work area length
*
         END   ,
